@@ -98,54 +98,10 @@ struct kmscon_terminal {
 	struct kmscon_pointer pointer;
 };
 
-static void do_clear_margins(struct screen *scr)
-{
-	unsigned int w, h, sw, sh;
-	struct uterm_mode *mode;
-	struct tsm_screen_attr attr;
-	int dh, dw;
-
-	mode = uterm_display_get_current(scr->disp);
-	if (!mode)
-		return;
-
-	sw = uterm_mode_get_width(mode);
-	sh = uterm_mode_get_height(mode);
-
-	tsm_vte_get_def_attr(scr->term->vte, &attr);
-
-	if (scr->txt->orientation == OR_NORMAL || scr->txt->orientation == OR_UPSIDE_DOWN) {
-		w = FONT_WIDTH(scr->txt) * scr->txt->cols;
-		h = FONT_HEIGHT(scr->txt) * scr->txt->rows;
-	} else {
-		w = FONT_HEIGHT(scr->txt) * scr->txt->rows;
-		h = FONT_WIDTH(scr->txt) * scr->txt->cols;
-	}
-	dw = sw - w;
-	dh = sh - h;
-
-	if (dw) {
-		if (scr->txt->orientation == OR_NORMAL || scr->txt->orientation == OR_LEFT)
-			uterm_display_fill(scr->disp, attr.br, attr.bg, attr.bb,
-				w, 0,dw, sh);
-		else
-			uterm_display_fill(scr->disp, attr.br, attr.bg, attr.bb,
-				0, 0,dw, sh);
-	}
-	if (dh) {
-		if (scr->txt->orientation == OR_NORMAL || scr->txt->orientation == OR_RIGHT)
-			uterm_display_fill(scr->disp, attr.br, attr.bg, attr.bb,
-				0, h, sw, dh);
-		else
-		 	uterm_display_fill(scr->disp, attr.br, attr.bg, attr.bb,
-				0, 0, sw, dh);
-	}
-}
-
 static int font_set(struct kmscon_terminal *term);
 
-
-static void coord_to_cell(struct kmscon_terminal *term, int32_t x, int32_t y, unsigned int *posx, unsigned int *posy)
+static void coord_to_cell(struct kmscon_terminal *term, int32_t x, int32_t y, unsigned int *posx,
+			  unsigned int *posy)
 {
 	int fw = term->font->attr.width;
 	int fh = term->font->attr.height;
@@ -156,51 +112,41 @@ static void coord_to_cell(struct kmscon_terminal *term, int32_t x, int32_t y, un
 	*posy = y / fh;
 
 	if (*posx >= w)
-		*posx = w;
+		*posx = w - 1;
 
 	if (*posy >= h)
-		*posy = h;
+		*posy = h - 1;
 }
 
 static void draw_pointer(struct screen *scr)
 {
-	struct tsm_screen_attr attr;
-
 	if (!scr->term->pointer.visible)
 		return;
 
-	tsm_vte_get_def_attr(scr->term->vte, &attr);
-	kmscon_text_draw_pointer(scr->txt, scr->term->pointer.x, scr->term->pointer.y, &attr);
+	kmscon_text_draw_pointer(scr->txt, scr->term->pointer.x, scr->term->pointer.y);
 }
 
 static void do_redraw_screen(struct screen *scr)
 {
+	struct tsm_screen_attr attr;
 	int ret;
 
 	if (!scr->term->awake || !kmscon_session_get_foreground(scr->term->session))
 		return;
 
 	scr->pending = false;
-	do_clear_margins(scr);
 
-	kmscon_text_prepare(scr->txt);
+	tsm_vte_get_def_attr(scr->term->vte, &attr);
+	kmscon_text_prepare(scr->txt, &attr);
 	tsm_screen_draw(scr->term->console, kmscon_text_draw_cb, scr->txt);
 	draw_pointer(scr);
 	kmscon_text_render(scr->txt);
 
-	ret = uterm_display_swap(scr->disp, false);
-
-	if (ret == -EAGAIN) {
-		uterm_display_deactivate(scr->disp);
-		ret = uterm_display_activate(scr->disp, NULL);
-		if (!ret)
-			ret = font_set(scr->term);
-		if (!ret)
-			ret = uterm_display_swap(scr->disp, false);
-	}
-
+	ret = uterm_display_swap(scr->disp);
 	if (ret) {
-		log_warning("cannot swap display %p", scr->disp);
+		if (ret != -EBUSY)
+			log_warning("cannot swap display [%s] %d", uterm_display_name(scr->disp),
+				    ret);
 		return;
 	}
 
@@ -226,7 +172,8 @@ static void redraw_all(struct kmscon_terminal *term)
 	if (!term->awake)
 		return;
 
-	shl_dlist_for_each(iter, &term->screens) {
+	shl_dlist_for_each(iter, &term->screens)
+	{
 		scr = shl_dlist_entry(iter, struct screen, list);
 		redraw_screen(scr);
 	}
@@ -237,7 +184,8 @@ static bool has_kms_display(struct kmscon_terminal *term)
 	struct shl_dlist *iter;
 	struct screen *scr;
 
-	shl_dlist_for_each(iter, &term->screens) {
+	shl_dlist_for_each(iter, &term->screens)
+	{
 		scr = shl_dlist_entry(iter, struct screen, list);
 		if (uterm_display_is_drm(scr->disp))
 			return true;
@@ -254,7 +202,6 @@ static void update_pointer_max_all(struct kmscon_terminal *term)
 {
 	struct shl_dlist *iter;
 	struct screen *scr;
-	struct uterm_mode *mode;
 	unsigned int max_x = INT_MAX;
 	unsigned int max_y = INT_MAX;
 	unsigned int sw, sh;
@@ -262,19 +209,16 @@ static void update_pointer_max_all(struct kmscon_terminal *term)
 	if (!term->awake)
 		return;
 
-	shl_dlist_for_each(iter, &term->screens) {
+	shl_dlist_for_each(iter, &term->screens)
+	{
 		scr = shl_dlist_entry(iter, struct screen, list);
 
-		mode = uterm_display_get_current(scr->disp);
-		if (!mode)
-			continue;
-
 		if (scr->txt->orientation == OR_NORMAL || scr->txt->orientation == OR_UPSIDE_DOWN) {
-			sw = uterm_mode_get_width(mode);
-			sh = uterm_mode_get_height(mode);
+			sw = uterm_display_get_width(scr->disp);
+			sh = uterm_display_get_height(scr->disp);
 		} else {
-			sw = uterm_mode_get_height(mode);
-			sh = uterm_mode_get_width(mode);
+			sw = uterm_display_get_height(scr->disp);
+			sh = uterm_display_get_width(scr->disp);
 		}
 		if (!sw || !sh)
 			continue;
@@ -296,7 +240,8 @@ static void redraw_all_test(struct kmscon_terminal *term)
 	if (!term->awake)
 		return;
 
-	shl_dlist_for_each(iter, &term->screens) {
+	shl_dlist_for_each(iter, &term->screens)
+	{
 		scr = shl_dlist_entry(iter, struct screen, list);
 		if (uterm_display_is_swapping(scr->disp))
 			scr->swapping = true;
@@ -304,8 +249,7 @@ static void redraw_all_test(struct kmscon_terminal *term)
 	}
 }
 
-static void display_event(struct uterm_display *disp,
-			  struct uterm_display_event *ev, void *data)
+static void display_event(struct uterm_display *disp, struct uterm_display_event *ev, void *data)
 {
 	struct screen *scr = data;
 
@@ -317,16 +261,14 @@ static void display_event(struct uterm_display *disp,
 		do_redraw_screen(scr);
 }
 
-static void osc_event(struct tsm_vte *vte, const char *osc_string,
-	size_t osc_len, void *data)
+static void osc_event(struct tsm_vte *vte, const char *osc_string, size_t osc_len, void *data)
 {
 	struct kmscon_terminal *term = data;
 
 	if (strcmp(osc_string, "setBackground") == 0) {
 		log_info("Got OSC setBackground");
 		kmscon_session_set_background(term->session);
-	}
-	else if (strcmp(osc_string, "setForeground") == 0) {
+	} else if (strcmp(osc_string, "setForeground") == 0) {
 		log_info("Got OSC setForeground");
 		kmscon_session_set_foreground(term->session);
 	}
@@ -344,41 +286,46 @@ static void mouse_event(struct tsm_vte *vte, enum tsm_mouse_track_mode track_mod
 /*
  * Resize terminal
  * We support multiple monitors per terminal. As some software-rendering
- * backends to not support scaling, we always use the smallest cols/rows that are
+ * backends do not support scaling, we always use the smallest cols/rows that are
  * provided so wider displays will have black margins.
- * This can be extended to support scaling but that would mean we need to check
- * whether the text-renderer backend supports that, first (TODO).
- *
- * If @force is true, then the console/pty are notified even though the size did
- * not changed. If @notify is false, then console/pty are not notified even
- * though the size might have changed. force = true and notify = false doesn't
- * make any sense, though.
  */
-static void terminal_resize(struct kmscon_terminal *term,
-			    unsigned int cols, unsigned int rows,
-			    bool force, bool notify)
+static bool terminal_update_size(struct kmscon_terminal *term)
 {
-	bool resize = false;
+	struct shl_dlist *iter;
+	struct screen *scr;
+	unsigned int min_cols = UINT_MAX;
+	unsigned int min_rows = UINT_MAX;
 
-	update_pointer_max_all(term);
+	shl_dlist_for_each(iter, &term->screens)
+	{
+		unsigned int cols, rows;
+		scr = shl_dlist_entry(iter, struct screen, list);
+		cols = kmscon_text_get_cols(scr->txt);
+		if (cols && cols < min_cols)
+			min_cols = cols;
 
-	if (!term->min_cols || (cols > 0 && cols < term->min_cols)) {
-		term->min_cols = cols;
-		resize = true;
+		rows = kmscon_text_get_rows(scr->txt);
+		if (rows && rows < min_rows)
+			min_rows = rows;
 	}
-	if (!term->min_rows || (rows > 0 && rows < term->min_rows)) {
-		term->min_rows = rows;
-		resize = true;
+	if (min_cols == UINT_MAX || min_rows == UINT_MAX)
+		return false;
+
+	if (min_cols == term->min_cols && min_rows == term->min_rows)
+		return false;
+
+	term->min_cols = min_cols;
+	term->min_rows = min_rows;
+	return true;
+}
+
+static void terminal_update_size_notify(struct kmscon_terminal *term)
+{
+	if (terminal_update_size(term)) {
+		tsm_screen_resize(term->console, term->min_cols, term->min_rows);
+		kmscon_pty_resize(term->pty, term->min_cols, term->min_rows);
+		redraw_all(term);
 	}
-
-	if (!notify || (!resize && !force))
-		return;
-	if (!term->min_cols || !term->min_rows)
-		return;
-
-	tsm_screen_resize(term->console, term->min_cols, term->min_rows);
-	kmscon_pty_resize(term->pty, term->min_cols, term->min_rows);
-	redraw_all(term);
 }
 
 static int font_set(struct kmscon_terminal *term)
@@ -389,14 +336,12 @@ static int font_set(struct kmscon_terminal *term)
 	struct screen *ent;
 
 	term->font_attr.bold = false;
-	ret = kmscon_font_find(&font, &term->font_attr,
-			       term->conf->font_engine);
+	ret = kmscon_font_find(&font, &term->font_attr, term->conf->font_engine);
 	if (ret)
 		return ret;
 
 	term->font_attr.bold = true;
-	ret = kmscon_font_find(&bold_font, &term->font_attr,
-			       term->conf->font_engine);
+	ret = kmscon_font_find(&bold_font, &term->font_attr, term->conf->font_engine);
 	if (ret) {
 		log_warning("cannot create bold font: %d", ret);
 		bold_font = font;
@@ -410,21 +355,15 @@ static int font_set(struct kmscon_terminal *term)
 
 	term->min_cols = 0;
 	term->min_rows = 0;
-	shl_dlist_for_each(iter, &term->screens) {
+	shl_dlist_for_each(iter, &term->screens)
+	{
 		ent = shl_dlist_entry(iter, struct screen, list);
 
 		ret = kmscon_text_set(ent->txt, font, bold_font, ent->disp);
 		if (ret)
-			log_warning("cannot change text-renderer font: %d",
-				    ret);
-
-		terminal_resize(term,
-				kmscon_text_get_cols(ent->txt),
-				kmscon_text_get_rows(ent->txt),
-				false, false);
+			log_warning("cannot change text-renderer font: %d", ret);
 	}
-
-	terminal_resize(term, 0, 0, true, true);
+	terminal_update_size_notify(term);
 	return 0;
 }
 
@@ -440,16 +379,13 @@ static void rotate_cw_all(struct kmscon_terminal *term)
 	struct shl_dlist *iter;
 	struct screen *scr;
 
-	shl_dlist_for_each(iter, &term->screens) {
+	shl_dlist_for_each(iter, &term->screens)
+	{
 		scr = shl_dlist_entry(iter, struct screen, list);
 		rotate_cw_screen(scr);
-		term->min_cols = 0;
-		term->min_rows = 0;
-		terminal_resize(term,
-				kmscon_text_get_cols(scr->txt),
-				kmscon_text_get_rows(scr->txt),
-				true, true);
 	}
+	terminal_update_size_notify(term);
+	update_pointer_max_all(term);
 }
 
 static void rotate_ccw_screen(struct screen *scr)
@@ -467,16 +403,13 @@ static void rotate_ccw_all(struct kmscon_terminal *term)
 	struct shl_dlist *iter;
 	struct screen *scr;
 
-	shl_dlist_for_each(iter, &term->screens) {
+	shl_dlist_for_each(iter, &term->screens)
+	{
 		scr = shl_dlist_entry(iter, struct screen, list);
 		rotate_ccw_screen(scr);
-		term->min_cols = 0;
-		term->min_rows = 0;
-		terminal_resize(term,
-				kmscon_text_get_cols(scr->txt),
-				kmscon_text_get_rows(scr->txt),
-				true, true);
 	}
+	terminal_update_size_notify(term);
+	update_pointer_max_all(term);
 }
 
 static int add_display(struct kmscon_terminal *term, struct uterm_display *disp)
@@ -487,7 +420,8 @@ static int add_display(struct kmscon_terminal *term, struct uterm_display *disp)
 	const char *be;
 	bool opengl;
 
-	shl_dlist_for_each(iter, &term->screens) {
+	shl_dlist_for_each(iter, &term->screens)
+	{
 		scr = shl_dlist_entry(iter, struct screen, list);
 		if (scr->disp == disp)
 			return 0;
@@ -508,10 +442,8 @@ static int add_display(struct kmscon_terminal *term, struct uterm_display *disp)
 		goto err_free;
 	}
 
-	ret = uterm_display_use(scr->disp, &opengl);
-	if (term->conf->render_engine)
-		be = term->conf->render_engine;
-	else if (ret >= 0 && opengl)
+	opengl = uterm_display_has_opengl(scr->disp);
+	if (opengl)
 		be = "gltex";
 	else
 		be = "bbulk";
@@ -522,26 +454,22 @@ static int add_display(struct kmscon_terminal *term, struct uterm_display *disp)
 		goto err_cb;
 	}
 
-	ret = kmscon_text_set(scr->txt, term->font, term->bold_font,
-			      scr->disp);
+	ret = kmscon_text_set(scr->txt, term->font, term->bold_font, scr->disp);
 	if (ret) {
 		log_error("cannot set text-renderer parameters");
 		goto err_text;
 	}
 
-	terminal_resize(term,
-			kmscon_text_get_cols(scr->txt),
-			kmscon_text_get_rows(scr->txt),
-			false, true);
-
 	shl_dlist_link(&term->screens, &scr->list);
 
-	log_notice("Using video backend [%s] with text renderer [%s] and font engine [%s]",
-		   uterm_display_backend_name(disp), scr->txt->ops->name,
+	log_notice("Display [%s] with backend [%s] text renderer [%s] font engine [%s]\n",
+		   uterm_display_name(disp), uterm_display_backend_name(disp), scr->txt->ops->name,
 		   term->font->ops->name);
 
 	log_debug("added display %p to terminal %p", disp, term);
-	redraw_screen(scr);
+
+	terminal_update_size_notify(term);
+	update_pointer_max_all(term);
 	uterm_display_ref(scr->disp);
 	return 0;
 
@@ -556,8 +484,6 @@ err_free:
 
 static void free_screen(struct screen *scr, bool update)
 {
-	struct shl_dlist *iter;
-	struct screen *ent;
 	struct kmscon_terminal *term = scr->term;
 
 	log_debug("destroying terminal screen %p", scr);
@@ -570,17 +496,8 @@ static void free_screen(struct screen *scr, bool update)
 	if (!update)
 		return;
 
-	term->min_cols = 0;
-	term->min_rows = 0;
-	shl_dlist_for_each(iter, &term->screens) {
-		ent = shl_dlist_entry(iter, struct screen, list);
-		terminal_resize(term,
-				kmscon_text_get_cols(ent->txt),
-				kmscon_text_get_rows(ent->txt),
-				false, false);
-	}
-
-	terminal_resize(term, 0, 0, true, true);
+	update_pointer_max_all(term);
+	terminal_update_size_notify(term);
 }
 
 static void rm_display(struct kmscon_terminal *term, struct uterm_display *disp)
@@ -588,7 +505,8 @@ static void rm_display(struct kmscon_terminal *term, struct uterm_display *disp)
 	struct shl_dlist *iter;
 	struct screen *scr;
 
-	shl_dlist_for_each(iter, &term->screens) {
+	shl_dlist_for_each(iter, &term->screens)
+	{
 		scr = shl_dlist_entry(iter, struct screen, list);
 		if (scr->disp == disp)
 			break;
@@ -601,45 +519,42 @@ static void rm_display(struct kmscon_terminal *term, struct uterm_display *disp)
 	free_screen(scr, true);
 }
 
-static void input_event(struct uterm_input *input,
-			struct uterm_input_key_event *ev,
-			void *data)
+static void input_event(struct uterm_input *input, struct uterm_input_key_event *ev, void *data)
 {
 	struct kmscon_terminal *term = data;
 
-	if (!term->opened || !term->awake || ev->handled || !kmscon_session_get_foreground(term->session))
+	if (!term->opened || !term->awake || ev->handled ||
+	    !kmscon_session_get_foreground(term->session))
 		return;
 
-	if (conf_grab_matches(term->conf->grab_scroll_up,
-			      ev->mods, ev->num_syms, ev->keysyms)) {
+	// reset mouse selection on keypress
+	tsm_screen_selection_reset(term->console);
+
+	if (conf_grab_matches(term->conf->grab_scroll_up, ev->mods, ev->num_syms, ev->keysyms)) {
 		tsm_screen_sb_up(term->console, 1);
 		redraw_all(term);
 		ev->handled = true;
 		return;
 	}
-	if (conf_grab_matches(term->conf->grab_scroll_down,
-			      ev->mods, ev->num_syms, ev->keysyms)) {
+	if (conf_grab_matches(term->conf->grab_scroll_down, ev->mods, ev->num_syms, ev->keysyms)) {
 		tsm_screen_sb_down(term->console, 1);
 		redraw_all(term);
 		ev->handled = true;
 		return;
 	}
-	if (conf_grab_matches(term->conf->grab_page_up,
-			      ev->mods, ev->num_syms, ev->keysyms)) {
+	if (conf_grab_matches(term->conf->grab_page_up, ev->mods, ev->num_syms, ev->keysyms)) {
 		tsm_screen_sb_page_up(term->console, 1);
 		redraw_all(term);
 		ev->handled = true;
 		return;
 	}
-	if (conf_grab_matches(term->conf->grab_page_down,
-			      ev->mods, ev->num_syms, ev->keysyms)) {
+	if (conf_grab_matches(term->conf->grab_page_down, ev->mods, ev->num_syms, ev->keysyms)) {
 		tsm_screen_sb_page_down(term->console, 1);
 		redraw_all(term);
 		ev->handled = true;
 		return;
 	}
-	if (conf_grab_matches(term->conf->grab_zoom_in,
-			      ev->mods, ev->num_syms, ev->keysyms)) {
+	if (conf_grab_matches(term->conf->grab_zoom_in, ev->mods, ev->num_syms, ev->keysyms)) {
 		ev->handled = true;
 		if (term->font_attr.points + 1 < term->font_attr.points)
 			return;
@@ -649,8 +564,7 @@ static void input_event(struct uterm_input *input,
 			--term->font_attr.points;
 		return;
 	}
-	if (conf_grab_matches(term->conf->grab_zoom_out,
-			      ev->mods, ev->num_syms, ev->keysyms)) {
+	if (conf_grab_matches(term->conf->grab_zoom_out, ev->mods, ev->num_syms, ev->keysyms)) {
 		ev->handled = true;
 		if (term->font_attr.points <= 1)
 			return;
@@ -660,14 +574,12 @@ static void input_event(struct uterm_input *input,
 			++term->font_attr.points;
 		return;
 	}
-	if (conf_grab_matches(term->conf->grab_rotate_cw,
-				ev->mods, ev->num_syms, ev->keysyms)) {
+	if (conf_grab_matches(term->conf->grab_rotate_cw, ev->mods, ev->num_syms, ev->keysyms)) {
 		rotate_cw_all(term);
 		ev->handled = true;
 		return;
 	}
-	if (conf_grab_matches(term->conf->grab_rotate_ccw,
-				ev->mods, ev->num_syms, ev->keysyms)) {
+	if (conf_grab_matches(term->conf->grab_rotate_ccw, ev->mods, ev->num_syms, ev->keysyms)) {
 		rotate_ccw_all(term);
 		ev->handled = true;
 		return;
@@ -679,8 +591,8 @@ static void input_event(struct uterm_input *input,
 	if (ev->num_syms > 1)
 		return;
 
-	if (tsm_vte_handle_keyboard(term->vte, ev->keysyms[0], ev->ascii,
-				    ev->mods, ev->codepoints[0])) {
+	if (tsm_vte_handle_keyboard(term->vte, ev->keysyms[0], ev->ascii, ev->mods,
+				    ev->codepoints[0])) {
 		tsm_screen_sb_reset(term->console);
 		redraw_all(term);
 		ev->handled = true;
@@ -708,13 +620,21 @@ static void copy_selection(struct kmscon_terminal *term)
 	term->pointer.copy_len = tsm_screen_selection_copy(term->console, &term->pointer.copy);
 }
 
-static void forward_pointer_event(struct kmscon_terminal *term, struct uterm_input_pointer_event *ev)
+static void forward_pointer_event(struct kmscon_terminal *term,
+				  struct uterm_input_pointer_event *ev)
 {
 	unsigned int event;
+	unsigned int button;
+
+	button = ev->button;
 
 	switch (ev->event) {
 	case UTERM_MOVED:
 		event = TSM_MOUSE_EVENT_MOVED;
+		/* In mouse tracking protocol, motion with button pressed uses button+32 */
+		if (ev->pressed && button <= 2) {
+			button += 32;
+		}
 		break;
 	case UTERM_BUTTON:
 		if (ev->pressed)
@@ -722,25 +642,36 @@ static void forward_pointer_event(struct kmscon_terminal *term, struct uterm_inp
 		else
 			event = TSM_MOUSE_EVENT_RELEASED;
 		break;
+	case UTERM_WHEEL:
+		/* Convert wheel events to button 4 (scroll up) or 5 (scroll down) */
+		event = TSM_MOUSE_EVENT_PRESSED;
+		if (ev->wheel > 0)
+			button = 4; /* Scroll up */
+		else
+			button = 5; /* Scroll down */
+		break;
 	default:
 		return;
 	}
-	tsm_vte_handle_mouse(term->vte, term->pointer.posx, term->pointer.posy,
-			term->pointer.x, term->pointer.y, ev->button, event, 0);
+	tsm_vte_handle_mouse(term->vte, term->pointer.posx, term->pointer.posy, term->pointer.x,
+			     term->pointer.y, button, event, 0);
 }
 
-static void handle_pointer_button(struct kmscon_terminal *term, struct uterm_input_pointer_event *ev)
+static void handle_pointer_button(struct kmscon_terminal *term,
+				  struct uterm_input_pointer_event *ev)
 {
-	switch(ev->button) {
+	switch (ev->button) {
 	case 0:
 		if (ev->pressed) {
 			if (ev->double_click) {
-				tsm_screen_selection_word(term->console, term->pointer.posx, term->pointer.posy);
+				tsm_screen_selection_word(term->console, term->pointer.posx,
+							  term->pointer.posy);
 				copy_selection(term);
 				term->pointer.select = false;
 			} else {
 				term->pointer.select = true;
-				start_selection(term->console, term->pointer.posx, term->pointer.posy);
+				start_selection(term->console, term->pointer.posx,
+						term->pointer.posy);
 			}
 		} else {
 			if (term->pointer.select)
@@ -754,14 +685,14 @@ static void handle_pointer_button(struct kmscon_terminal *term, struct uterm_inp
 		break;
 	case 2:
 		if (ev->pressed) {
-			kmscon_pty_write(term->pty, term->pointer.copy, term->pointer.copy_len);
+			if (term->pointer.copy && term->pointer.copy_len)
+				tsm_vte_paste(term->vte, term->pointer.copy);
 			tsm_screen_selection_reset(term->console);
 		}
 	}
 }
 
-static void pointer_event(struct uterm_input *input,
-			  struct uterm_input_pointer_event *ev,
+static void pointer_event(struct uterm_input *input, struct uterm_input_pointer_event *ev,
 			  void *data)
 {
 	struct kmscon_terminal *term = data;
@@ -770,7 +701,8 @@ static void pointer_event(struct uterm_input *input,
 		term->pointer.x = ev->pointer_x;
 		term->pointer.y = ev->pointer_y;
 
-		coord_to_cell(term, term->pointer.x, term->pointer.y, &term->pointer.posx, &term->pointer.posy);
+		coord_to_cell(term, term->pointer.x, term->pointer.y, &term->pointer.posx,
+			      &term->pointer.posy);
 		term->pointer.visible = true;
 	}
 
@@ -785,21 +717,23 @@ static void pointer_event(struct uterm_input *input,
 		break;
 	case UTERM_MOVED:
 		if (term->pointer.select)
-			update_selection(term->console,term->pointer.posx, term->pointer.posy);
+			update_selection(term->console, term->pointer.posx, term->pointer.posy);
 		break;
 	case UTERM_BUTTON:
 		handle_pointer_button(term, ev);
 		break;
 	case UTERM_WHEEL:
+		tsm_screen_selection_reset(term->console);
 		if (ev->wheel > 0)
 			tsm_screen_sb_up(term->console, 3);
 		else
-		 	tsm_screen_sb_down(term->console, 3);
+			tsm_screen_sb_down(term->console, 3);
 		break;
 	case UTERM_SYNC:
 		redraw_all(term);
 		break;
 	case UTERM_HIDE_TIMEOUT:
+		tsm_screen_selection_reset(term->console);
 		term->pointer.visible = false;
 		break;
 	}
@@ -866,8 +800,8 @@ static void terminal_destroy(struct kmscon_terminal *term)
 	free(term);
 }
 
-static int session_event(struct kmscon_session *session,
-			 struct kmscon_session_event *ev, void *data)
+static int session_event(struct kmscon_session *session, struct kmscon_session_event *ev,
+			 void *data)
 {
 	struct kmscon_terminal *term = data;
 
@@ -898,8 +832,7 @@ static int session_event(struct kmscon_session *session,
 	return 0;
 }
 
-static void pty_input(struct kmscon_pty *pty, const char *u8, size_t len,
-								void *data)
+static void pty_input(struct kmscon_pty *pty, const char *u8, size_t len, void *data)
 {
 	struct kmscon_terminal *term = data;
 
@@ -919,16 +852,15 @@ static void pty_event(struct ev_fd *fd, int mask, void *data)
 	kmscon_pty_dispatch(term->pty);
 }
 
-static void write_event(struct tsm_vte *vte, const char *u8, size_t len,
-			void *data)
+static void write_event(struct tsm_vte *vte, const char *u8, size_t len, void *data)
 {
 	struct kmscon_terminal *term = data;
 
 	kmscon_pty_write(term->pty, u8, len);
 }
 
-int kmscon_terminal_register(struct kmscon_session **out,
-			     struct kmscon_seat *seat, unsigned int vtnr)
+int kmscon_terminal_register(struct kmscon_session **out, struct kmscon_seat *seat,
+			     unsigned int vtnr)
 {
 	struct kmscon_terminal *term;
 	int ret;
@@ -949,8 +881,7 @@ int kmscon_terminal_register(struct kmscon_session **out,
 	term->conf_ctx = kmscon_seat_get_conf(seat);
 	term->conf = conf_ctx_get_mem(term->conf_ctx);
 
-	strncpy(term->font_attr.name, term->conf->font_name,
-		KMSCON_FONT_MAX_NAME - 1);
+	strncpy(term->font_attr.name, term->conf->font_name, KMSCON_FONT_MAX_NAME - 1);
 	term->font_attr.ppi = term->conf->font_ppi;
 	term->font_attr.points = term->conf->font_size;
 
@@ -959,13 +890,11 @@ int kmscon_terminal_register(struct kmscon_session **out,
 		goto err_free;
 	tsm_screen_set_max_sb(term->console, term->conf->sb_size);
 
-	ret = tsm_vte_new(&term->vte, term->console, write_event, term,
-			  log_llog, NULL);
+	ret = tsm_vte_new(&term->vte, term->console, write_event, term, log_llog, NULL);
 	if (ret)
 		goto err_con;
 
-	tsm_vte_set_backspace_sends_delete(term->vte,
-					   BUILD_BACKSPACE_SENDS_DELETE);
+	tsm_vte_set_backspace_sends_delete(term->vte, term->conf->backspace_delete);
 
 	tsm_vte_set_osc_cb(term->vte, osc_event, (void *)term);
 	tsm_vte_set_mouse_cb(term->vte, mouse_event, (void *)term);
@@ -986,33 +915,14 @@ int kmscon_terminal_register(struct kmscon_session **out,
 	if (ret)
 		goto err_font;
 
-	kmscon_pty_set_env_reset(term->pty, term->conf->reset_env);
-
-	ret = kmscon_pty_set_term(term->pty, term->conf->term);
+	ret = kmscon_pty_set_conf(term->pty, term->conf->term, "kmscon", term->conf->argv,
+				  kmscon_seat_get_name(seat), vtnr, term->conf->reset_env,
+				  term->conf->backspace_delete);
 	if (ret)
 		goto err_pty;
 
-	ret = kmscon_pty_set_colorterm(term->pty, "kmscon");
-	if (ret)
-		goto err_pty;
-
-	ret = kmscon_pty_set_argv(term->pty, term->conf->argv);
-	if (ret)
-		goto err_pty;
-
-	ret = kmscon_pty_set_seat(term->pty, kmscon_seat_get_name(seat));
-	if (ret)
-		goto err_pty;
-
-	if (vtnr > 0) {
-		ret = kmscon_pty_set_vtnr(term->pty, vtnr);
-		if (ret)
-			goto err_pty;
-	}
-
-	ret = ev_eloop_new_fd(term->eloop, &term->ptyfd,
-			      kmscon_pty_get_fd(term->pty),
-			      EV_READABLE, pty_event, term);
+	ret = ev_eloop_new_fd(term->eloop, &term->ptyfd, kmscon_pty_get_fd(term->pty), EV_READABLE,
+			      pty_event, term);
 	if (ret)
 		goto err_pty;
 
@@ -1026,8 +936,7 @@ int kmscon_terminal_register(struct kmscon_session **out,
 			goto err_input;
 	}
 
-	ret = kmscon_seat_register_session(seat, &term->session, session_event,
-					   term);
+	ret = kmscon_seat_register_session(seat, &term->session, session_event, term);
 	if (ret) {
 		log_error("cannot register session for terminal: %d", ret);
 		goto err_pointer;
