@@ -69,13 +69,13 @@ struct kmscon_pty {
 	char *seat;
 	char *vtnr;
 	bool env_reset;
+	bool backspace_delete;
 
 	time_t last_spawn_time;
 	int retry_count;
 };
 
-int kmscon_pty_new(struct kmscon_pty **out, kmscon_pty_input_cb input_cb,
-		   void *data)
+int kmscon_pty_new(struct kmscon_pty **out, kmscon_pty_input_cb input_cb, void *data)
 {
 	struct kmscon_pty *pty;
 	int ret;
@@ -139,94 +139,41 @@ void kmscon_pty_unref(struct kmscon_pty *pty)
 	free(pty);
 }
 
-int kmscon_pty_set_term(struct kmscon_pty *pty, const char *term)
+int kmscon_pty_set_conf(struct kmscon_pty *pty, const char *term, const char *colorterm,
+			char **argv, const char *seat, unsigned int vtnr, bool do_reset,
+			bool backspace)
 {
-	char *t;
-
-	if (!pty || !term)
-		return -EINVAL;
-
-	t = strdup(term);
-	if (!t)
-		return -ENOMEM;
-	free(pty->term);
-	pty->term = t;
-
-	return 0;
-}
-
-int kmscon_pty_set_colorterm(struct kmscon_pty *pty, const char *colorterm)
-{
-	char *t;
-
-	if (!pty || !colorterm)
-		return -EINVAL;
-
-	t = strdup(colorterm);
-	if (!t)
-		return -ENOMEM;
-	free(pty->colorterm);
-	pty->colorterm = t;
-
-	return 0;
-}
-
-int kmscon_pty_set_argv(struct kmscon_pty *pty, char **argv)
-{
-	char **t;
-	int ret;
-
-	if (!pty || !argv || !*argv || !**argv)
-		return -EINVAL;
-
-	ret = shl_dup_array(&t, argv);
-	if (ret)
-		return ret;
-
-	free(pty->argv);
-	pty->argv = t;
-	return 0;
-}
-
-int kmscon_pty_set_seat(struct kmscon_pty *pty, const char *seat)
-{
-	char *t;
-
-	if (!pty || !seat)
-		return -EINVAL;
-
-	t = strdup(seat);
-	if (!t)
-		return -ENOMEM;
-	free(pty->seat);
-	pty->seat = t;
-
-	return 0;
-}
-
-int kmscon_pty_set_vtnr(struct kmscon_pty *pty, unsigned int vtnr)
-{
-	char *t;
-	int ret;
+	char *vt;
 
 	if (!pty)
 		return -EINVAL;
 
-	ret = asprintf(&t, "%u", vtnr);
-	if (ret < 0)
-		return -ENOMEM;
-	free(pty->vtnr);
-	pty->vtnr = t;
-
-	return 0;
-}
-
-void kmscon_pty_set_env_reset(struct kmscon_pty *pty, bool do_reset)
-{
-	if (!pty)
-		return;
-
 	pty->env_reset = do_reset;
+	pty->backspace_delete = backspace;
+
+	if (term) {
+		pty->term = strdup(term);
+		if (!pty->term)
+			return -ENOMEM;
+	}
+	if (colorterm) {
+		pty->colorterm = strdup(colorterm);
+		if (!pty->colorterm)
+			return -ENOMEM;
+	}
+	if (seat) {
+		pty->seat = strdup(seat);
+		if (!pty->seat)
+			return -ENOMEM;
+	}
+	if (vtnr) {
+		if (asprintf(&vt, "%u", vtnr) < 0)
+			return -ENOMEM;
+		pty->vtnr = vt;
+	}
+	if (argv && *argv && **argv)
+		return shl_dup_array(&pty->argv, argv);
+	return 0;
 }
 
 int kmscon_pty_get_fd(struct kmscon_pty *pty)
@@ -250,27 +197,28 @@ static bool pty_is_open(struct kmscon_pty *pty)
 	return pty->fd >= 0;
 }
 
-static void __attribute__((noreturn))
-exec_child(const char *term, const char *colorterm, char **argv,
-	   const char *seat, const char *vtnr, bool env_reset, bool drm)
+static void __attribute__((noreturn)) exec_child(const char *term, const char *colorterm,
+						 char **argv, const char *seat, const char *vtnr,
+						 bool env_reset, bool drm)
 {
 	char **env;
 	char **def_argv;
+	char **argv_parsed;
+	int ret;
 
 	if (env_reset) {
-		env = malloc(sizeof(char*));
+		env = malloc(sizeof(char *));
 		if (!env) {
-			log_error("cannot allocate memory for environment (%d): %m",
-				  errno);
+			log_error("cannot allocate memory for environment (%d): %m", errno);
 			exit(EXIT_FAILURE);
 		}
 
-		memset(env, 0, sizeof(char*));
+		memset(env, 0, sizeof(char *));
 		environ = env;
 
-		def_argv = (char*[]){ "/bin/login", "-p", NULL };
+		def_argv = (char *[]){"/bin/login", "-p", NULL};
 	} else {
-		def_argv = (char*[]){ "/bin/login", NULL };
+		def_argv = (char *[]){"/bin/login", NULL};
 	}
 
 	if (!term)
@@ -292,15 +240,18 @@ exec_child(const char *term, const char *colorterm, char **argv,
 	} else {
 		setenv("TERM_SESSION_TYPE", "fb", 1);
 	}
+	ret = shl_replace_array_with_env(&argv_parsed, argv);
+	if (ret)
+		argv_parsed = argv;
 
-	execve(argv[0], argv, environ);
+	execve(argv_parsed[0], argv_parsed, environ);
 
 	log_err("failed to exec child %s: %m", argv[0]);
 
 	exit(EXIT_FAILURE);
 }
 
-static void setup_child(int master, struct winsize *ws)
+static void setup_child(int master, bool backspace_delete, struct winsize *ws)
 {
 	int ret;
 	sigset_t sigset;
@@ -356,7 +307,7 @@ static void setup_child(int master, struct winsize *ws)
 		goto err_out;
 	}
 
-	if (BUILD_BACKSPACE_SENDS_DELETE) {
+	if (backspace_delete) {
 		/* erase character should be delete */
 		attr.c_cc[VERASE] = 0177;
 	} else {
@@ -377,8 +328,8 @@ static void setup_child(int master, struct winsize *ws)
 	}
 
 	if (dup2(slave, STDIN_FILENO) != STDIN_FILENO ||
-			dup2(slave, STDOUT_FILENO) != STDOUT_FILENO ||
-			dup2(slave, STDERR_FILENO) != STDERR_FILENO) {
+	    dup2(slave, STDOUT_FILENO) != STDOUT_FILENO ||
+	    dup2(slave, STDERR_FILENO) != STDERR_FILENO) {
 		log_err("cannot duplicate slave: %m");
 		goto err_out;
 	}
@@ -400,8 +351,8 @@ err_out:
  * a little bit more control of the process, and as a bonus avoid linking to
  * the libutil library in glibc.
  */
-static int pty_spawn(struct kmscon_pty *pty, int master,
-		     unsigned short width, unsigned short height, bool drm)
+static int pty_spawn(struct kmscon_pty *pty, int master, unsigned short width,
+		     unsigned short height, bool drm)
 {
 	pid_t pid;
 	struct winsize ws;
@@ -416,9 +367,9 @@ static int pty_spawn(struct kmscon_pty *pty, int master,
 		log_err("cannot fork: %m");
 		return -errno;
 	case 0:
-		setup_child(master, &ws);
-		exec_child(pty->term, pty->colorterm, pty->argv, pty->seat,
-			   pty->vtnr, pty->env_reset, drm);
+		setup_child(master, pty->backspace_delete, &ws);
+		exec_child(pty->term, pty->colorterm, pty->argv, pty->seat, pty->vtnr,
+			   pty->env_reset, drm);
 		exit(EXIT_FAILURE);
 	default:
 		log_debug("forking child %d", pid);
@@ -444,8 +395,7 @@ static int send_buf(struct kmscon_pty *pty)
 		}
 
 		if (ret < 0 && errno != EWOULDBLOCK) {
-			log_warn("cannot write to child process (%d): %m",
-				 errno);
+			log_warn("cannot write to child process (%d): %m", errno);
 			return ret;
 		}
 
@@ -472,12 +422,10 @@ static int read_buf(struct kmscon_pty *pty)
 			if (pty->input_cb)
 				pty->input_cb(pty, pty->io_buf, len, pty->data);
 		} else if (len == 0) {
-			log_debug("HUP during read on pty of child %d",
-				  pty->child);
+			log_debug("HUP during read on pty of child %d", pty->child);
 			break;
 		} else if (errno != EWOULDBLOCK) {
-			log_debug("cannot read from pty of child %d (%d): %m",
-				  pty->child, errno);
+			log_debug("cannot read from pty of child %d (%d): %m", pty->child, errno);
 			break;
 		}
 	} while (len > 0 && --num);
@@ -526,8 +474,7 @@ static void pty_input(struct ev_fd *fd, int mask, void *data)
 		read_buf(pty);
 }
 
-static void sig_child(struct ev_eloop *eloop, struct ev_child_data *chld,
-			void *data)
+static void sig_child(struct ev_eloop *eloop, struct ev_child_data *chld, void *data)
 {
 	struct kmscon_pty *pty = data;
 	time_t current_time;
@@ -535,8 +482,7 @@ static void sig_child(struct ev_eloop *eloop, struct ev_child_data *chld,
 	if (chld->pid != pty->child)
 		return;
 
-	log_info("child exited: pid: %u status: %d",
-		 chld->pid, chld->status);
+	log_info("child exited: pid: %u status: %d", chld->pid, chld->status);
 
 	if (pty->retry_count == MAX_RETRY_COUNT) {
 		log_err("reached max retry attempts for login process");
@@ -555,8 +501,7 @@ static void sig_child(struct ev_eloop *eloop, struct ev_child_data *chld,
 	pty->input_cb(pty, NULL, 0, pty->data);
 }
 
-int kmscon_pty_open(struct kmscon_pty *pty, unsigned short width,
-		    unsigned short height, bool drm)
+int kmscon_pty_open(struct kmscon_pty *pty, unsigned short width, unsigned short height, bool drm)
 {
 	int ret;
 	int master;
@@ -573,8 +518,7 @@ int kmscon_pty_open(struct kmscon_pty *pty, unsigned short width,
 		return -errno;
 	}
 
-	ret = ev_eloop_new_fd(pty->eloop, &pty->efd, master,
-			      EV_ET | EV_READABLE, pty_input, pty);
+	ret = ev_eloop_new_fd(pty->eloop, &pty->efd, master, EV_ET | EV_READABLE, pty_input, pty);
 	if (ret)
 		goto err_master;
 
@@ -659,8 +603,7 @@ void kmscon_pty_signal(struct kmscon_pty *pty, int signum)
 	log_debug("send signal %d to child", signum);
 }
 
-void kmscon_pty_resize(struct kmscon_pty *pty,
-			unsigned short width, unsigned short height)
+void kmscon_pty_resize(struct kmscon_pty *pty, unsigned short width, unsigned short height)
 {
 	int ret;
 	struct winsize ws;
